@@ -7,7 +7,6 @@ UNUserNotificationCenter backend for macOS.
 * Only available from signed app bundles if called from the main executable or from a
   signed Python framework (for example from python.org).
 * Requires a running CFRunLoop to invoke callbacks.
-
 """
 
 # system imports
@@ -16,14 +15,17 @@ import logging
 import enum
 import asyncio
 from concurrent.futures import Future
-from typing import Optional, cast
+from urllib.parse import urlparse, unquote
+from typing import cast, Optional
 
 # external imports
-from rubicon.objc import NSObject, ObjCClass, objc_method, py_from_ns  # type: ignore
-from rubicon.objc.runtime import load_library, objc_id, objc_block  # type: ignore
+from packaging.version import Version
+from rubicon.objc import NSObject, ObjCClass, objc_method, py_from_ns
+from rubicon.objc.runtime import load_library, objc_id, objc_block
 
 # local imports
-from .base import Notification, DesktopNotifierBase, AuthorisationError
+from .base import Notification, DesktopNotifierBase, AuthorisationError, Urgency
+from .macos_support import macos_version
 
 
 __all__ = ["CocoaNotificationCenter"]
@@ -41,9 +43,11 @@ UNTextInputNotificationAction = ObjCClass("UNTextInputNotificationAction")
 UNNotificationCategory = ObjCClass("UNNotificationCategory")
 UNNotificationSound = ObjCClass("UNNotificationSound")
 UNNotificationAttachment = ObjCClass("UNNotificationAttachment")
+UNNotificationSettings = ObjCClass("UNNotificationSettings")
 
 NSURL = ObjCClass("NSURL")
 NSSet = ObjCClass("NSSet")
+NSError = ObjCClass("NSError")
 
 # UserNotifications.h
 
@@ -85,14 +89,20 @@ class UNErrorCode(enum.Enum):
     NotificationInvalidNoContent = 1401
 
 
+class UNNotificationInterruptionLevel(enum.Enum):
+    Passive = 0
+    Active = 1
+    TimeSensitive = 2
+    Critical = 3
+
+
 class NotificationCenterDelegate(NSObject):  # type: ignore
     """Delegate to handle user interactions with notifications"""
 
-    @objc_method
+    @objc_method  # type:ignore
     def userNotificationCenter_didReceiveNotificationResponse_withCompletionHandler_(
         self, center, response, completion_handler: objc_block
     ) -> None:
-
         # Get the notification which was clicked from the platform ID.
         platform_nid = py_from_ns(response.notification.request.identifier)
         py_notification = self.interface._notification_for_nid[platform_nid]
@@ -102,23 +112,19 @@ class NotificationCenterDelegate(NSObject):  # type: ignore
 
         # Invoke the callback which corresponds to the user interaction.
         if response.actionIdentifier == UNNotificationDefaultActionIdentifier:
-
             if py_notification.on_clicked:
                 py_notification.on_clicked()
 
         elif response.actionIdentifier == UNNotificationDismissActionIdentifier:
-
             if py_notification.on_dismissed:
                 py_notification.on_dismissed()
 
         elif response.actionIdentifier == ReplyActionIdentifier:
-
             if py_notification.reply_field.on_replied:
                 reply_text = py_from_ns(response.userText)
                 py_notification.reply_field.on_replied(reply_text)
 
         else:
-
             button_number = int(py_from_ns(response.actionIdentifier))
             callback = py_notification.buttons[button_number].on_pressed
 
@@ -137,19 +143,22 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
     :param app_name: The name of the app. Does not have any effect because the app
         name is automatically determined from the bundle or framework.
-    :param app_icon: The icon of the app. Does not have any effect because the app
-        icon is automatically determined from the bundle or framework.
     :param notification_limit: Maximum number of notifications to keep in the system's
         notification center.
     """
 
+    _to_native_urgency = {
+        Urgency.Low: UNNotificationInterruptionLevel.Passive,
+        Urgency.Normal: UNNotificationInterruptionLevel.Active,
+        Urgency.Critical: UNNotificationInterruptionLevel.TimeSensitive,
+    }
+
     def __init__(
         self,
         app_name: str = "Python",
-        app_icon: Optional[str] = None,
         notification_limit: Optional[int] = None,
     ) -> None:
-        super().__init__(app_name, app_icon, notification_limit)
+        super().__init__(app_name, notification_limit)
         self.nc = UNUserNotificationCenter.currentNotificationCenter()
         self.nc_delegate = NotificationCenterDelegate.alloc().init()
         self.nc_delegate.interface = self
@@ -167,7 +176,7 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         :returns: Whether authorisation has been granted.
         """
 
-        future: Future = Future()
+        future: Future[tuple[bool, str]] = Future()
 
         def on_auth_completed(granted: bool, error: objc_id) -> None:
             ns_error = py_from_ns(error)
@@ -193,7 +202,7 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
         # Get existing notification categories.
 
-        future: Future = Future()
+        future: Future[UNNotificationSettings] = Future()  # type:ignore
 
         def handler(settings: objc_id) -> None:
             settings = py_from_ns(settings)
@@ -204,13 +213,13 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
         settings = await asyncio.wrap_future(future)
 
-        authorized = settings.authorizationStatus in (
+        authorized = settings.authorizationStatus in (  # type:ignore
             UNAuthorizationStatusAuthorized,
             UNAuthorizationStatusProvisional,
             UNAuthorizationStatusEphemeral,
         )
 
-        settings.release()
+        settings.release()  # type:ignore
 
         return authorized
 
@@ -231,22 +240,25 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         else:
             platform_nid = str(uuid.uuid4())
 
-        # On macOS, we need need to register a new notification category for every
+        # On macOS, we need to register a new notification category for every
         # unique set of buttons.
         category_id = await self._create_category_for_notification(notification)
 
-        # Create the native notification + notification request.
+        # Create the native notification and notification request.
         content = UNMutableNotificationContent.alloc().init()
         content.title = notification.title
         content.body = notification.message
         content.categoryIdentifier = category_id
         content.threadIdentifier = notification.thread
+        if macos_version >= Version("12.0"):
+            content.interruptionLevel = self._to_native_urgency[notification.urgency]
 
         if notification.sound:
             content.sound = UNNotificationSound.defaultSound
 
         if notification.attachment:
-            url = NSURL.fileURLWithPath(notification.attachment)
+            path = unquote(urlparse(notification.attachment).path)
+            url = NSURL.fileURLWithPath(path, isDirectory=False)
             attachment = UNNotificationAttachment.attachmentWithIdentifier(
                 "", URL=url, options={}, error=None
             )
@@ -256,7 +268,7 @@ class CocoaNotificationCenter(DesktopNotifierBase):
             platform_nid, content=content, trigger=None
         )
 
-        future: Future = Future()
+        future: Future[NSError] = Future()  # type:ignore
 
         def handler(error: objc_id) -> None:
             ns_error = py_from_ns(error)
@@ -270,28 +282,30 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         )
 
         # Error handling.
-
         error = await asyncio.wrap_future(future)
 
         if error:
-            error.autorelease()
+            error.autorelease()  # type:ignore
 
-            if error.domain == UNErrorDomain:
-                if error.code == UNErrorCode.NotificationsNotAllowed:
+            if error.domain == UNErrorDomain:  # type:ignore
+                if error.code == UNErrorCode.NotificationsNotAllowed:  # type:ignore
                     raise AuthorisationError("Not authorised")
-                elif error.code == UNErrorCode.NotificationInvalidNoDate:
+                elif error.code == UNErrorCode.NotificationInvalidNoDate:  # type:ignore
                     raise RuntimeError("Missing notification date")
-                elif error.code == UNErrorCode.NotificationInvalidNoContent:
+                elif (
+                    error.code  # type:ignore
+                    == UNErrorCode.NotificationInvalidNoContent
+                ):
                     raise RuntimeError("Missing notification content")
                 else:
                     # In case of attachment errors, the notification will still be
                     # delivered, just without an attachment. We therefore do not raise
                     # the error.
                     logger.warning(
-                        f"{error.localizedDescription}: {notification.attachment}"
+                        f"{error.localizedDescription}: {notification.attachment}"  # type:ignore
                     )
             else:
-                raise RuntimeError(error.localizedDescription)
+                raise RuntimeError(error.localizedDescription)  # type:ignore
 
         return platform_nid
 
@@ -321,7 +335,6 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
         # Register new category if necessary.
         if category_id not in category_ids:
-
             # Create action for each button.
             actions = []
 
@@ -355,10 +368,10 @@ class CocoaNotificationCenter(DesktopNotifierBase):
 
         return category_id
 
-    async def _get_notification_categories(self) -> NSSet:  # type: ignore
+    async def _get_notification_categories(self) -> NSSet:  # type:ignore
         """Returns the registered notification categories for this app / Python."""
 
-        future: Future = Future()
+        future: Future[NSSet] = Future()  # type:ignore
 
         def handler(categories: objc_id) -> None:
             categories = py_from_ns(categories)
@@ -368,7 +381,7 @@ class CocoaNotificationCenter(DesktopNotifierBase):
         self.nc.getNotificationCategoriesWithCompletionHandler(handler)
 
         categories = await asyncio.wrap_future(future)
-        categories.autorelease()
+        categories.autorelease()  # type:ignore
 
         return categories
 
